@@ -1,25 +1,25 @@
       subroutine getvofh(bw,kern,wght,vol)
       implicit logical(a-z)
       integer kern
-      real*8 bw,wght(2),vol,sofw3D
+      double precision bw,wght(2),vol,sofw3D
       external sofw3D
       vol=sofw3D(bw,kern,wght)
       RETURN
       END
-      real*8 function sofw3D(bw,kern,wght)
+      double precision function sofw3D(bw,kern,wght)
       implicit logical(a-z)
       integer kern
-      real*8 bw,wght(2)
+      double precision bw,wght(2)
       integer j1,j2,j3,dlw1,dlw2,dlw3,clw1,clw2,clw3,ih1,ih2,ih3
-      real*8 sw,sw2,h2,lkern,z1,z2,z3,z
+      double precision sw,sw2,h2,lkern,z1,z2,z3,z
       external lkern
       h2=bw*bw
 C
 C   first calculate location weights
 C
-      ih3=bw/wght(2)
-      ih2=bw/wght(1)
-      ih1=bw
+      ih3=FLOOR(bw/wght(2))
+      ih2=FLOOR(bw/wght(1))
+      ih1=FLOOR(bw)
       dlw1=2*ih1+1
       dlw2=2*ih2+1
       dlw3=2*ih3+1
@@ -31,11 +31,11 @@ C
       DO j3=1,dlw3
          z3=(clw3-j3)*wght(2)
          z3=z3*z3
-         ih2=sqrt(h2-z3)/wght(1)
+         ih2=FLOOR(sqrt(h2-z3)/wght(1))
          DO j2=clw2-ih2,clw2+ih2
             z2=(clw2-j2)*wght(1)
             z2=z3+z2*z2
-            ih1=sqrt(h2-z2)
+            ih1=FLOOR(sqrt(h2-z2))
             DO j1=clw1-ih1,clw1+ih1
                z1=clw1-j1
                z=lkern(kern,(z1*z1+z2)/h2)
@@ -50,26 +50,26 @@ C
       subroutine sofw3Df(bw,kern,wght,fw)
       implicit logical(a-z)
       integer kern
-      real*8 bw,wght(2),fw,sofw3D
+      double precision bw,wght(2),fw,sofw3D
       external sofw3D
       fw=sofw3D(bw,kern,wght)
       RETURN
       END
       
-      real*8 function sofw3D0(bw,kern,wght)
+      double precision function sofw3D0(bw,kern,wght)
       implicit logical(a-z)
       integer kern
-      real*8 bw,wght(2)
+      double precision bw,wght(2)
       integer j1,j2,j3,dlw1,dlw2,dlw3,clw1,clw2,clw3,ih1,ih2,ih3
-      real*8 sw,h2,lkern,z1,z2,z3,z
+      double precision sw,h2,lkern,z1,z2,z3,z
       external lkern
       h2=bw*bw
 C
 C   first calculate location weights
 C
-      ih3=bw/wght(2)
-      ih2=bw/wght(1)
-      ih1=bw
+      ih3=FLOOR(bw/wght(2))
+      ih2=FLOOR(bw/wght(1))
+      ih1=FLOOR(bw)
       dlw1=2*ih1+1
       dlw2=2*ih2+1
       dlw3=2*ih3+1
@@ -80,11 +80,11 @@ C
       DO j3=1,dlw3
          z3=(clw3-j3)*wght(2)
          z3=z3*z3
-         ih2=sqrt(h2-z3)/wght(1)
+         ih2=FLOOR(sqrt(h2-z3)/wght(1))
          DO j2=clw2-ih2,clw2+ih2
             z2=(clw2-j2)*wght(1)
             z2=z3+z2*z2
-            ih1=sqrt(h2-z2)
+            ih1=FLOOR(sqrt(h2-z2))
             DO j1=clw1-ih1,clw1+ih1
                z1=clw1-j1
                z=lkern(kern,(z1*z1+z2)/h2)
@@ -98,8 +98,8 @@ C
       subroutine ni2var(bw,kern,wght,quot)
       implicit logical(a-z)
       integer kern
-      real*8 bw,wght(2),quot
-      real*8 sofw3D0,sofw3D
+      double precision bw,wght(2),quot
+      double precision sofw3D0,sofw3D
       external sofw3D0,sofw3D
       quot=sofw3D0(bw,kern,wght)/sofw3D(bw,kern,wght)
       RETURN
@@ -114,9 +114,9 @@ C  Algorithmus zur Nullstellenbestimmung einer monotonen Funktion auf(0,\infty)
       subroutine gethani(x,y,kern,value,wght,eps,bw)
       implicit logical(a-z)
       integer kern
-      real*8 x,y,value,wght(2),eps,bw
-      real*8 fw1,fw2,fw3,z
-      real*8 sofw3D
+      double precision x,y,value,wght(2),eps,bw
+      double precision fw1,fw2,fw3,z
+      double precision sofw3D
       external sofw3D
       if(x.ge.y) RETURN
       fw1=sofw3D(x,kern,wght)
@@ -155,16 +155,16 @@ C         z=x+(value-fw1)/(fw2-fw1)*(y-x)
 
       implicit logical(a-z)
       integer n1,n2,n3,nv,lag(3)
-      real*8 scorr,res(nv,n1,n2,n3)
+      double precision scorr,res(nv,n1,n2,n3)
       logical mask(n1,n2,n3)
-      real*8 z2,y2,resi,resip1,vrm,vrmp1,zk,zcorr,z
+      double precision z2,y2,resi,resip1,vrm,vrmp1,zk,zcorr,z
       integer i1,i2,i3,i4,l1,l2,l3,k
       zk=nv
       l1=lag(1)
       l2=lag(2)
       l3=lag(3)
       z=0.d0
-      k=0.d0
+      k=0
 C  correlation in x
       do i1=1,n1-l1
          do i2=1,n2-l2
@@ -193,14 +193,73 @@ C  correlation in x
       scorr=z/k
       return
       end
+      subroutine mcorlag1(res,mask,indm,nvox,n1,n2,n3,nv,scorr,lag)
+      implicit logical(a-z)
+      integer n1,n2,n3,nv,lag(3),nvox,indm(nvox)
+      double precision scorr,res(nv,nvox)
+      logical mask(n1,n2,n3)
+      double precision z2,y2,resi,resip1,vrm,vrmp1,zk,zcorr,z
+      integer i1,i2,i3,i4,l1,l2,l3,k,i,j,m,m0
+      zk=nv
+      l1=lag(1)
+      l2=lag(2)
+      l3=lag(3)
+      z=0.d0
+      k=0
+C  correlation in x
+      do j=1,nvox
+         i=indm(j)
+         i3=i/n1/n2+1
+         i=i-(i3-1)*n1*n2
+         if(i.eq.0) THEN
+            i=n1*n2
+            i3=i3-1
+         ENDIF
+         i2=i/n1+1
+         i1=i-(i2-1)*n1
+         if(i1.eq.0) THEN
+            i1=n1
+            i2=i2-1
+         ENDIF
+         if(i3.gt.n3-l3) CYCLE
+         if(i2.gt.n2-l2) CYCLE
+         if(i1.gt.n1-l1) CYCLE
+         if (.not.mask(i1+l1,i2+l2,i3+l3)) CYCLE
+         z2=0.d0
+         y2=0.d0
+         zcorr=0.d0
+C   get position in indm corresponding to voxel i1+l1,i2+l2,i3+l3
+         i=i1+l1+(i2+l2-1)*n1+(i3+l3-1)*n1*n2
+         DO m=1,nvox
+            m0=m
+            if(i.eq.indm(m)) EXIT
+         END DO
+         do i4=1,nv
+            resi=res(i4,j)
+            resip1=res(i4,m0)
+            z2=z2+resi*resi
+            y2=y2+resip1*resip1
+            zcorr=zcorr+resi*resip1
+         enddo
+         vrm=z2/zk
+         vrmp1=y2/zk
+         vrm=vrm*vrmp1
+         if(vrm.gt.1e-10) THEN
+            z=z+zcorr/zk/sqrt(vrm)
+            k=k+1
+         end if
+      enddo
+      scorr=z/k
+      return
+      end
       subroutine sweepm(res,mask,n1,n2,n3,nv)
 
       implicit logical(a-z)
       integer n1,n2,n3,nv
-      real*8 res(nv,n1,n2,n3)
+      double precision res(nv,n1,n2,n3)
       logical mask(n1,n2,n3)
       integer i1,i2,i3,k
-      real*8 z
+      double precision z
       Do i1=1,n1
          Do i2=1,n2
             Do i3=1,n3
@@ -218,13 +277,32 @@ C  correlation in x
       END DO
       return
       end
+      subroutine sweepm0(res,n,nv)
+
+      implicit logical(a-z)
+      integer n,nv
+      double precision res(nv,n)
+      integer i,k
+      double precision z
+      Do i=1,n
+         z=0.d0
+         DO k=1,nv
+            z=z+res(k,i)
+         END DO
+         z=z/nv
+         DO k=1,nv
+            res(k,i)=res(k,i)-z
+         END DO
+      END DO
+      return
+      end
       subroutine mean3D(res,n1,n2,n3,nv,mres)
 
       implicit logical(a-z)
       integer n1,n2,n3,nv
-      real*8 res(nv,n1,n2,n3),mres(n1,n2,n3)
+      double precision res(nv,n1,n2,n3),mres(n1,n2,n3)
       integer i1,i2,i3,k
-      real*8 z
+      double precision z
       Do i1=1,n1
          Do i2=1,n2
             Do i3=1,n3
@@ -242,7 +320,7 @@ C  correlation in x
 
       implicit logical(a-z)
       integer n1,n2,n3,nv,l1,l2,l3,lag(3)
-      real*8 scorr(l1,l2,l3),res(nv,n1,n2,n3)
+      double precision scorr(l1,l2,l3),res(nv,n1,n2,n3)
       logical mask(n1,n2,n3)
       integer i1,i2,i3
       Do i1=1,l1
@@ -258,20 +336,40 @@ C  correlation in x
       END DO
       return
       end
+      subroutine mcorr1(res,mask,indm,nvox,n1,n2,n3,nv,scorr,l1,l2,l3)
+      implicit logical(a-z)
+      integer n1,n2,n3,nv,l1,l2,l3,lag(3),nvox,indm(nvox)
+      double precision scorr(l1,l2,l3),res(nv,n1,n2,n3)
+      logical mask(n1,n2,n3)
+      integer i1,i2,i3
+      Do i1=1,l1
+         lag(1)=i1-1
+         DO i2=1,l2
+            lag(2)=i2-1
+            DO i3=1,l3
+               lag(3)=i3-1
+               call mcorlag1(res,mask,indm,nvox,n1,n2,n3,nv,
+     1                       scorr(i1,i2,i3),lag)
+               call rchkusr()  
+            END DO
+         END DO
+      END DO
+      return
+      end
       subroutine imcorrl(res,mask,n1,n2,n3,nv,scorr,lag)
 
       implicit logical(a-z)
       integer n1,n2,n3,nv,lag(3)
-      real*8 scorr,res(nv,n1,n2,n3)
+      double precision scorr,res(nv,n1,n2,n3)
       logical mask(n1,n2,n3)
-      real*8 z2,y2,resi,resip1,vrm,vrmp1,zk,zcorr,z
+      double precision z2,y2,resi,resip1,vrm,vrmp1,zk,zcorr,z
       integer i1,i2,i3,i4,l1,l2,l3,k
       zk=nv
       l1=lag(1)
       l2=lag(2)
       l3=lag(3)
       z=0.d0
-      k=0.d0
+      k=0
 C  correlation in x
       do i1=1,n1-l1
          do i2=1,n2-l2
@@ -304,7 +402,7 @@ C  correlation in x
 
       implicit logical(a-z)
       integer n1,n2,n3,nv,l1,l2,l3,lag(3)
-      real*8 scorr(l1,l2,l3),res(nv,n1,n2,n3)
+      double precision scorr(l1,l2,l3),res(nv,n1,n2,n3)
       logical mask(n1,n2,n3)
       integer i1,i2,i3
       Do i1=1,l1
@@ -324,9 +422,9 @@ C  correlation in x
 
       implicit logical(a-z)
       integer n1,n2,n3,l1,l2,l3,lag(3)
-      real*8 scorr(l1,l2,l3),w(n1,n2,n3)
+      double precision scorr(l1,l2,l3),w(n1,n2,n3)
       integer i1,i2,i3
-      real*8 z,zcorr
+      double precision z,zcorr
       z=0.d0
       DO i1=1,n1
          DO i2=1,n2
@@ -353,9 +451,9 @@ C  correlation in x
 
       implicit logical(a-z)
       integer n1,n2,n3,lag(3)
-      real*8 scorr,w(n1,n2,n3)
+      double precision scorr,w(n1,n2,n3)
       integer i1,i2,i3,c1,c2,c3,j1,j2,j3,l1,l2,l3
-      real*8 z
+      double precision z
       c1=(n1-1)/2
       c2=(n2-1)/2
       c3=(n3-1)/2
@@ -386,9 +484,9 @@ C   compute variance estimates !!! (not the inverse)
 C
       implicit logical(a-z)
       integer n1,n2,n3,nv
-      real*8 resscale,var(n1,n2,n3),res(nv,n1,n2,n3)
+      double precision resscale,var(n1,n2,n3),res(nv,n1,n2,n3)
       logical mask(n1,n2,n3)
-      real*8 z2,zk,resi,ressc2,z1
+      double precision z2,zk,resi,ressc2,z1
       integer i1,i2,i3,i4
       zk=nv
       ressc2=resscale*resscale
